@@ -16,10 +16,6 @@
  *  License along with this library; if not, write to the Free
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
-/*
- * $Id: rexxsaa.h,v 1.13 2002/03/09 01:20:38 mark Exp $
- */
-
 #ifndef __REXXSAA_H_INCLUDED
 #define __REXXSAA_H_INCLUDED
 /* Remember to add prototypes for Rexx*() functions */
@@ -35,18 +31,27 @@
 #if defined(_MSC_VER) && !defined(__WINS__)
 # if _MSC_VER >= 1100
 /* Stupid MSC can't compile own headers without warning at least in VC 5.0 */
-#   pragma warning(disable: 4115 4201 4214)
+#   pragma warning(disable: 4115 4201 4214 4514)
 # endif
 # include <windows.h>
 # if _MSC_VER >= 1100
 #   pragma warning(default: 4115 4201 4214)
 # endif
+# define VOID_TYPEDEFED
 #endif
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+/*
+ * pack(1) was introduced in Regina 3.3 to become binary compatible in
+ * structures with OS/2, Object Rexx, etc.
+ */
+#ifdef OREXX_BINARY_COMPATIBLE
+# pragma pack(1)
+#endif
+
 #ifndef CONST
 /* Introduced in Regina 2.0, may/should not exist in non-ANSI compilers.
  * Everybody can see which params will change even if CONST is not defined.
@@ -58,49 +63,84 @@ extern "C" {
 #endif
 
 #if defined(__MINGW32__)
-typedef CONST char *PCSZ ;
-#endif
-
-#if defined(__MINGW32__) && defined(__REGINA_DLL)
-#define EXTNAME(n) __attribute__((alias (n)))
+# if defined(__REGINA_DLL)
+#  define EXTNAME(n) __declspec(dllexport)
+# else
+#  define EXTNAME(n) __declspec(dllimport)
+# endif
 #else
 #define EXTNAME(n)
 #endif
 
-#if !defined(_OS2EMX_H) && !defined(__RSXNT__) && !defined(__MINGW32__)
-typedef char CHAR ;
-typedef short SHORT ;
-typedef long LONG ;
-typedef char *PSZ ;
-typedef CONST char *PCSZ ;
-# ifndef VOID
+#if !defined(OS2_VAC_C) && !defined(_OS2EMX_H) && !defined(__RSXNT__) && !defined(__MINGW32__)
+
+# if !defined(VOID_TYPEDEFED) && !defined(VOID)
 typedef void VOID ;
+#  define VOID_TYPEDEFED
 # endif
+
+# ifndef PVOID_TYPEDEFED
 typedef void* PVOID ;
+#  define PVOID_TYPEDEFED
+# endif
 
+# ifndef CHAR_TYPEDEFED
+typedef char CHAR;
+#  define CHAR_TYPEDEFED
+# endif
+# ifndef PCHAR_TYPEDEFED
 typedef CHAR *PCHAR ;
-typedef SHORT *PSHORT ;
-typedef LONG *PLONG ;
-
+#  define PCHAR_TYPEDEFED
+# endif
 # ifndef UCHAR_TYPEDEFED
 typedef unsigned char UCHAR ;
 #  define UCHAR_TYPEDEFED
 # endif
+# ifndef PUCHAR_TYPEDEFED
+typedef UCHAR *PUCHAR;
+#  define PUCHAR_TYPEDEFED
+# endif
+
+# ifndef SHORT_TYPEDEFED
+typedef short SHORT;
+#  define SHORT_TYPEDEFED
+# endif
+# ifndef PSHORT_TYPEDEFED
+typedef SHORT *PSHORT ;
+#  define PSHORT_TYPEDEFED
+# endif
 # ifndef USHORT_TYPEDEFED
 typedef unsigned short USHORT ;
 #  define USHORT_TYPEDEFED
 # endif
+# ifndef PUSHORT_TYPEDEFED
+typedef USHORT *PUSHORT;
+#  define PUSHORT_TYPEDEFED
+# endif
+
+# ifndef LONG_TYPEDEFED
+typedef long LONG;
+#  define LONG_TYPEDEFED
+# endif
+# ifndef PLONG_TYPEDEFED
+typedef LONG *PLONG;
+#  define PLONG_TYPEDEFED
+# endif
 # ifndef ULONG_TYPEDEFED
 typedef unsigned long ULONG ;
 #  define ULONG_TYPEDEFED
 # endif
 
-typedef USHORT *PUSHORT ;
+typedef char *PSZ ;
+typedef CONST char *PCSZ ;
 typedef CHAR *PCH ;
-typedef UCHAR *PUCHAR ;
 
 #endif
 
+#ifdef __MINGW32__
+typedef CONST char *PCSZ ;
+#endif
+
 #ifdef INCL_REXXSAA
 # define INCL_RXSUBCOM
 # define INCL_RXSHV
@@ -112,6 +152,14 @@ typedef UCHAR *PUCHAR ;
 # define INCL_RXMACRO
 #endif
 
+/*
+ * For Innotek gcc, force use of _System calling convention
+ */
+#if defined( __EMX__ ) && defined( __INNOTEK_LIBC__ )
+# undef APIENTRY
+# define APIENTRY _System
+#endif
+
 #if !defined(APIENTRY)
 # define APIENTRY
 #endif
@@ -120,7 +168,7 @@ typedef UCHAR *PUCHAR ;
 # define APIRET ULONG
 #endif
 
-#if !defined(_OS2EMX_H)
+#if !defined(_OS2EMX_H) && !defined( PFN_TYPEDEFED )
 typedef APIRET (APIENTRY *PFN)();
 #endif
 
@@ -159,6 +207,15 @@ typedef struct {
 } RXSYSEXIT ;
 typedef RXSYSEXIT *PRXSYSEXIT ;
 
+typedef PUCHAR PEXIT ;
+
+/*
+ * typedefs for Rexx handler types
+ */
+typedef APIRET APIENTRY RexxFunctionHandler(PCSZ name, ULONG argc, /* CONST */ PRXSTRING argv, PCSZ queuename, PRXSTRING returnstring) ;
+typedef LONG APIENTRY RexxExitHandler( LONG, LONG, /* CONST */ PEXIT ) ;
+typedef APIRET APIENTRY RexxSubcomHandler(/* CONST */ PRXSTRING, PUSHORT, PRXSTRING);
+
 /*
  * -------------------------------------------------------------------
  * System Exit Interface
@@ -209,9 +266,11 @@ typedef RXSYSEXIT *PRXSYSEXIT ;
 #define RXENV       12  /* System Environment interface */
 # define RXENVGET    1  /* Get System Environment Variable */
 # define RXENVSET    2  /* Set System Environment Variable */
+# define RXCWDGET    3  /* Get Current Working Directory */
+# define RXCWDSET    4  /* Set Current Working Directory */
 
 #define RXENDLST     0
-#define RXNOOFEXITS 12
+#define RXNOOFEXITS 13  /* MUST be 1 more than last exit number */
 
 /* Symbolic return codes for System Exit Handlers */
 #define RXEXIT_HANDLED       0
@@ -272,8 +331,8 @@ typedef struct {
 
 typedef struct {
    struct {
-               unsigned rcfmlifo:1 ;
-   }                    rcmsq_flags ;
+               unsigned rxfmlifo:1 ;
+   }                    rxmsq_flags ;
    /* CONST */ RXSTRING rxmsq_value ;
 } RXMSQPSH_PARM ;
 
@@ -323,57 +382,20 @@ typedef struct {
    RXSTRING rxenv_value ;
 } RXENVSET_PARM ;
 
-/*
- #define rxfnc_flags    rxfnccal.u_rxfnc_flags
- #define rxfnc_name     rxfnccal.u_rxfnc_name
- #define rxfnc_namel    rxfnccal.u_rxfnc_namel
- #define rxfnc_que      rxfnccal.u_rxfnc_que
- #define rxfnc_quel     rxfnccal.u_rxfnc_quel
- #define rxfnc_argc     rxfnccal.u_rxfnc_argc
- #define rxfnc_argv     rxfnccal.u_rxfnc_argv
- #define rxfnc_retc     rxfnccal.u_rxfnc_retc
- #define rxcmd_flags    rxcmdhst.u_rxcmd_flags
- #define rxcmd_address  rxcmdhst.u_rxcmd_address
- #define rxcmd_addressl rxcmdhst.u_rxcmd_addressl
- #define rxcmd_dll      rxcmdhst.u_rxcmd_dll
- #define rxcmd_dll_len  rxcmdhst.u_rxcmd_dll_len
- #define rxcmd_command  rxcmdhst.u_rxcmd_command
- #define rxcmd_retc     rxcmdhst.u_rxcmd_retc
- #define rxmsq_retc     rxmsqpll.u_rxmsq_retc
- #define rxsio_string   rxsiosay.u_rxsio_string
- #define rxsiotrd_retc  rxsiotrd.u_rxsiotrd_retc
- #define rxsiodtr_retc  rxsiodtr.u_rxsiodtr_retc
- #define rxhlt_flags    rxhlttst.u_rxhlt_flags
- #define rxtrc_flags    rxtrctst.u_rxtrc_flags
- */
+typedef struct {
+   RXSTRING rxcwd_value ;
+} RXCWDGET_PARM ;
 
-typedef union {
-   RXFNCCAL_PARM fnccal ;
-   RXCMDHST_PARM cmdhst ;
-   RXMSQPLL_PARM msqpll ;
-   RXMSQPSH_PARM msqpsh ;
-   RXMSQSIZ_PARM msqsiz ;
-   RXMSQNAM_PARM msqnam ;
-   RXSIOSAY_PARM siosay ;
-   RXSIOTRC_PARM siotrc ;
-   RXSIOTRD_PARM siotrd ;
-   RXSIODTR_PARM siodtr ;
-   RXHLTTST_PARM hlttst ;
-   RXTRCTST_PARM trctst ;
-   RXENVGET_PARM envget ;
-   RXENVSET_PARM envset ;
-} EXIT ;
-
-typedef PUCHAR PEXIT ;
-
-typedef LONG APIENTRY RexxExitHandler( LONG, LONG, /* CONST */ PEXIT ) ;
+typedef struct {
+   RXSTRING rxcwd_value ;
+} RXCWDSET_PARM ;
 
 APIRET APIENTRY RexxRegisterExitExe(
                 PCSZ             EnvName,
-#ifdef RX_STRONGTYPING
-                RexxExitHandler *EntryPoint,
-#else
+#ifdef RX_WEAKTYPING
                 PFN              EntryPoint,
+#else
+                RexxExitHandler *EntryPoint,
 #endif
     /* CONST */ PUCHAR           UserArea )
 EXTNAME("RexxRegisterExitExe");
@@ -405,8 +427,7 @@ EXTNAME("RexxQueryExit");
 #endif /* INCL_RXSYSEXIT */
 
 
-
-#define MAXENVNAMELEN 31
+#define MAXENVNAMELEN 32767   /* at least, there is no limit */
 
 #define MAKERXSTRING(x,c,l)   ((x).strptr=(c),(x).strlength=(l))
 #define RXNULLSTRING(x)       (!(x).strptr)
@@ -515,9 +536,26 @@ EXTNAME("RexxStart");
 #define RX_START_OK         0
 #define RX_START_BADP       1  /* Bad parameters */
 #define RX_START_UNIMPL     2  /* Not yet implemented :-) */
-#define RX_START_TOOMANYP   3  /* To many parameters */
+#define RX_START_TOOMANYP   3  /* Too many parameters */
 #define RX_DIDNT_START      4  /* Unable to start interpreter */
 
+APIRET APIENTRY RexxCallBack(
+                PCSZ       ProcedureName,
+                LONG       ArgCount,
+                PRXSTRING  ArgList,
+                PSHORT     ReturnCode,
+                PRXSTRING  Result )
+EXTNAME("RexxCallBack");
+#define REXXCALLBACK RexxCallBack
+
+/*
+ * Return codes for RexxCallBack
+ */
+#define RX_CB_OK         0
+#define RX_CB_BADP       1  /* Bad parameters */
+#define RX_CB_NOTSTARTED 2  /* Interface not running */
+#define RX_CB_TOOMANYP   3  /* Too many parameters */
+#define RX_CB_BADN       8  /* Procedure not found */
 /*
  * -------------------------------------------------------------------
  * Sub-command Interface
@@ -525,14 +563,12 @@ EXTNAME("RexxStart");
  */
 #ifdef INCL_RXSUBCOM
 
-typedef APIRET APIENTRY RexxSubcomHandler(/* CONST */ PRXSTRING, PUSHORT, PRXSTRING);
-
 APIRET APIENTRY RexxRegisterSubcomExe(
                 PCSZ               EnvName,
-#ifdef RX_STRONGTYPING
-                RexxSubcomHandler *EntryPoint,
-#else
+#ifdef RX_WEAKTYPING
                 PFN                EntryPoint,
+#else
+                RexxSubcomHandler *EntryPoint,
 #endif
     /* CONST */ PUCHAR             UserArea )
 EXTNAME("RexxRegisterSubcomExe");
@@ -557,7 +593,7 @@ APIRET APIENTRY RexxQuerySubcom(
                 PCSZ    Envname,
                 PCSZ    ModuleName,
                 PUSHORT Flag,       /* Documentation diverges ... */
-                PUCHAR  UserArea ) 
+                PUCHAR  UserArea )
 EXTNAME("RexxQuerySubcom");
 #define REXXQUERYSUBCOM RexxQuerySubcom
 
@@ -602,14 +638,12 @@ EXTNAME("RexxQuerySubcom");
 #define RXFUNC_BADTYPE       70
 #define RXFUNC_NOEMEM      1002
 
-typedef APIRET APIENTRY RexxFunctionHandler(PCSZ name, ULONG argc, /* CONST */ PRXSTRING argv, PCSZ queuename, PRXSTRING returnstring) ;
-
 APIRET APIENTRY RexxRegisterFunctionExe(
                 PCSZ                 name,
-#ifdef RX_STRONGTYPING
-                RexxFunctionHandler *EntryPoint )
-#else
+#ifdef RX_WEAKTYPING
                 PFN                  EntryPoint )
+#else
+                RexxFunctionHandler *EntryPoint )
 #endif
 EXTNAME("RexxRegisterFunctionExe");
 #define REXXREGISTERFUNCTIONEXE RexxRegisterFunctionExe
@@ -652,6 +686,18 @@ APIRET APIENTRY RexxSetHalt(
 EXTNAME("RexxSetHalt");
 #define REXXSETHALT RexxSetHalt
 
+APIRET APIENTRY RexxSetTrace(
+                LONG pid,
+                LONG tid)
+EXTNAME("RexxSetTrace");
+#define REXXSETTRACE RexxSetTrace
+
+APIRET APIENTRY RexxResetTrace(
+                LONG pid,
+                LONG tid)
+EXTNAME("RexxResetTrace");
+#define REXXRESETTRACE RexxResetTrace
+
 #endif /* INCL_RXARI */
 
 /*
@@ -681,7 +727,7 @@ EXTNAME("RexxQueryQueue");
 #define REXXQUERYQUEUE RexxQueryQueue
 
 ULONG  APIENTRY RexxAddQueue (
-                PSZ,      
+                PSZ,
                 PRXSTRING,
                 ULONG )
 EXTNAME("RexxAddQueue");
@@ -718,7 +764,8 @@ EXTNAME("RexxPullQueue");
 #define RXQUEUE_ACCESS       10        /* Queue busy and wait active  */
 #define RXQUEUE_MAXREG       11        /* No memory to create a queue */
 #define RXQUEUE_MEMFAIL      12        /* Failure in memory management*/
-  
+#define RXQUEUE_NETERROR    100        /* Network error               */
+
 #endif /* INCL_RXQUEUE */
 
 /*
@@ -767,7 +814,7 @@ EXTNAME("RexxReorderMacro");
 #define REXXREORDERMACRO  RexxReorderMacro
 
 APIRET APIENTRY RexxClearMacroSpace(
-                VOID )
+                )
 EXTNAME("RexxClearMacroSpace");
 #define REXXCLEARMACROSPACE  RexxClearMacroSpace
 
@@ -791,10 +838,13 @@ EXTNAME("RexxClearMacroSpace");
 
 
 /* REGINA EXTENSIONS *********************************************************/
-/* The following function is an extension to the standard. Never try to
- * address the function directly. Use the dynamic linking machanism of
- * your operating system instead. This function was introduced in version
- * 2.0.
+/* The following functions are an extension to the standard. Never try to
+ * address the function directly if you want to be compatible. Use the dynamic
+ * linking machanism of your operating system instead.
+ *
+ *
+ * ReginaVersion returns informations about the version of the library.
+ * This function was introduced in version 2.0.
  * Returns: ULONG, in lower byte the two-digit fraction part of the version.
  *          The higher bytes will hold the integer part of the version.
  *          Examples: 0x10A codes the Version "1.10".
@@ -810,11 +860,33 @@ EXTNAME("RexxClearMacroSpace");
  *       although it is never counted in VersionString.strlength.
  *       RexxAllocateMemory is used if needed.
  */
-APIRET APIENTRY ReginaVersion( 
+APIRET APIENTRY ReginaVersion(
                 PRXSTRING VersionString )
 EXTNAME("ReginaVersion");
 #define REGINAVERSION ReginaVersion
 
+/*
+ * ReginaCleanup performs a graceful cleanup. This is done automatically
+ * on many systems but you can't be sure in all cases. The cleanup operations
+ * destroys all informations that are collected so far by one thread.
+ * It works for a single process or thread too. Although all memory is
+ * freed, some open handles may remain open. It is necessary to perform
+ * the appropriate "RexxDeregister<whatever>" calls before this routine
+ * is called.
+ * This function was introduced in version 3.3.
+ * Returns: ULONG, 0 if this call hasn't done anything.
+ *          1 if at least something could be freed.
+ * It is allowed to reuse every API function after this call but this routine
+ * must not be used when some parts of the Regina core are in use.
+ */
+APIRET APIENTRY ReginaCleanup( )
+EXTNAME("ReginaCleanup");
+#define REGINACLEANUP ReginaCleanup
+
+#ifdef OREXX_BINARY_COMPATIBLE
+# pragma pack()
+#endif
+
 #ifdef __cplusplus
 }
 #endif
